@@ -2,17 +2,52 @@
 # this repository contains the full copyright notices and license terms.
 import datetime
 from collections import defaultdict
+from functools import wraps
 
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.pyson import Eval
 from trytond.pool import Pool, PoolMeta
+from trytond.tools import grouped_slice
 from trytond.transaction import Transaction
 from trytond.modules.stock import StockMixin
 
 __all__ = ['Lot', 'LotType', 'Move', 'ShipmentIn', 'ShipmentOut',
     'LotByLocationContext', 'ShipmentOutReturn',
     'Period', 'PeriodCacheLot',
-    'Inventory', 'InventoryLine', 'LotByLocationContext']
+    'Inventory', 'InventoryLine', 'InventoryCount', 'InventoryCountSearch',
+    'LotByLocationContext']
+
+
+def check_no_move(func):
+    def find_moves(cls, records, state=None):
+        pool = Pool()
+        Move = pool.get('stock.move')
+        for sub_records in grouped_slice(records):
+            domain = [
+                ('lot', 'in', [r.id for r in sub_records])
+                ]
+            if state:
+                domain.append(('state', '=', state))
+            moves = Move.search(domain, limit=1, order=[])
+            if moves:
+                return True
+        return False
+
+    @wraps(func)
+    def decorator(cls, *args):
+        transaction = Transaction()
+        if (transaction.user != 0
+                and transaction.context.get('_check_access')):
+            actions = iter(args)
+            for records, values in zip(actions, actions):
+                for field, state, error in cls._modify_no_move:
+                    if field in values:
+                        if find_moves(cls, records, state):
+                            cls.raise_user_error(error)
+                        # No moves for those records
+                        break
+        func(cls, *args)
+    return decorator
 
 
 class Lot(ModelSQL, ModelView, StockMixin):
@@ -26,12 +61,23 @@ class Lot(ModelSQL, ModelView, StockMixin):
     forecast_quantity = fields.Function(fields.Float('Forecast Quantity'),
         'get_quantity', searcher='search_quantity')
 
+    @classmethod
+    def __setup__(cls):
+        super(Lot, cls).__setup__()
+        cls._error_messages.update({
+                'change_product': ("You cannot change the product of a lot "
+                    "which is associated to stock moves."),
+                })
+        cls._modify_no_move = [
+            ('product', None, 'change_product'),
+            ]
+
     @classmethod
     def get_quantity(cls, lots, name):
         location_ids = Transaction().context.get('locations')
-        products = list(set(l.product for l in lots))
-        return cls._get_quantity(lots, name, location_ids, products,
-            grouping=('product', 'lot'))
+        product_ids = list(set(l.product.id for l in lots))
+        return cls._get_quantity(lots, name, location_ids,
+            grouping=('product', 'lot',), grouping_filter=(product_ids,))
 
     @classmethod
     def search_quantity(cls, name, domain=None):
@@ -39,6 +85,11 @@ class Lot(ModelSQL, ModelView, StockMixin):
         return cls._search_quantity(name, location_ids, domain,
             grouping=('product', 'lot'))
 
+    @classmethod
+    @check_no_move
+    def write(cls, *args):
+        super(Lot, cls).write(*args)
+
 
 class LotByLocationContext(ModelView):
     'Lot by Location'
@@ -70,8 +121,7 @@ class LotType(ModelSQL, ModelView):
     code = fields.Char('Code', required=True)
 
 
-class Move:
-    __metaclass__ = PoolMeta
+class Move(metaclass=PoolMeta):
     __name__ = 'stock.move'
     lot = fields.Many2One('stock.lot', 'Lot',
         domain=[
@@ -105,8 +155,7 @@ class Move:
             move.check_lot()
 
 
-class ShipmentIn:
-    __metaclass__ = PoolMeta
+class ShipmentIn(metaclass=PoolMeta):
     __name__ = 'stock.shipment.in'
 
     @classmethod
@@ -117,8 +166,7 @@ class ShipmentIn:
         return move
 
 
-class ShipmentOut:
-    __metaclass__ = PoolMeta
+class ShipmentOut(metaclass=PoolMeta):
     __name__ = 'stock.shipment.out'
 
     @classmethod
@@ -146,10 +194,12 @@ class ShipmentOut:
                         round=False)
                     values = {}
                     if quantity < out_quantity:
-                        outgoing_moves.extend(Move.copy([out_move], default={
-                                    'quantity': out_move.uom.round(
-                                        out_quantity - quantity),
-                                    }))
+                        with Transaction().set_context(_stock_move_split=True):
+                            outgoing_moves.extend(
+                                Move.copy([out_move], default={
+                                        'quantity': out_move.uom.round(
+                                            out_quantity - quantity),
+                                        }))
                         values['quantity'] = out_move.uom.round(quantity)
                     values['lot'] = move.lot.id
                     to_write.extend(([out_move], values))
@@ -159,8 +209,7 @@ class ShipmentOut:
             Move.write(*to_write)
 
 
-class ShipmentOutReturn:
-    __metaclass__ = PoolMeta
+class ShipmentOutReturn(metaclass=PoolMeta):
     __name__ = 'stock.shipment.out.return'
 
     @classmethod
@@ -172,8 +221,7 @@ class ShipmentOutReturn:
         return move
 
 
-class Period:
-    __metaclass__ = PoolMeta
+class Period(metaclass=PoolMeta):
     __name__ = 'stock.period'
     lot_caches = fields.One2Many('stock.period.cache.lot', 'period',
         'Lot Caches', readonly=True)
@@ -209,8 +257,7 @@ class PeriodCacheLot(ModelSQL, ModelView):
     internal_quantity = fields.Float('Internal Quantity', readonly=True)
 
 
-class Inventory:
-    __metaclass__ = PoolMeta
+class Inventory(metaclass=PoolMeta):
     __name__ = 'stock.inventory'
 
     @classmethod
@@ -218,8 +265,7 @@ class Inventory:
         return super(Inventory, cls).grouping() + ('lot', )
 
 
-class InventoryLine:
-    __metaclass__ = PoolMeta
+class InventoryLine(metaclass=PoolMeta):
     __name__ = 'stock.inventory.line'
     lot = fields.Many2One('stock.lot', 'Lot',
         domain=[
@@ -246,3 +292,56 @@ class InventoryLine:
         if move:
             move.lot = self.lot
         return move
+
+
+class InventoryCount(metaclass=PoolMeta):
+    __name__ = 'stock.inventory.count'
+
+    @classmethod
+    def __setup__(cls):
+        super(InventoryCount, cls).__setup__()
+        cls._error_messages.update({
+                'lot_required': 'Only lot can be selected for "%(product)s".',
+                })
+
+    def default_quantity(self, fields):
+        pool = Pool()
+        Product = pool.get('product.product')
+        Inventory = pool.get('stock.inventory')
+        context = Transaction().context
+        inventory = Inventory(context['active_id'])
+        if isinstance(self.search.search, Product):
+            product = self.search.search
+            if product.lot_is_required(
+                    inventory.location, inventory.lost_found):
+                self.raise_user_error('lot_required', {
+                        'product': product.rec_name,
+                        })
+        return super(InventoryCount, self).default_quantity(fields)
+
+    def get_line_domain(self, inventory):
+        pool = Pool()
+        Lot = pool.get('stock.lot')
+        domain = super(InventoryCount, self).get_line_domain(inventory)
+        if isinstance(self.search.search, Lot):
+            domain.append(('lot', '=', self.search.search.id))
+        return domain
+
+    def get_line_values(self, inventory):
+        pool = Pool()
+        Lot = pool.get('stock.lot')
+        values = super(InventoryCount, self).get_line_values(inventory)
+        if isinstance(self.search.search, Lot):
+            lot = self.search.search
+            values['product'] = lot.product.id
+            values['lot'] = lot.id
+        return values
+
+
+class InventoryCountSearch(metaclass=PoolMeta):
+    __name__ = 'stock.inventory.count.search'
+
+    @classmethod
+    def __setup__(cls):
+        super(InventoryCountSearch, cls).__setup__()
+        cls.search.selection.append(('stock.lot', "Lot"))
